import { world, system, ItemStack, Vector } from "@minecraft/server";
import "./forms";

system.beforeEvents.watchdogTerminate.subscribe(data => data.cancel = true);

const players = Array.from(world.getPlayers());

world.afterEvents.playerPlaceBlock.subscribe(async scb => {
    const { block, player } = scb;
    const loc = block.location;
    const anyPlr = players.some(player => player.hasTag(`${loc.x} ${loc.y} ${loc.z}`));
    if (block.typeId.includes('scb:') && !anyPlr) {
        player.addTag(`${block.x} ${block.y} ${block.z}`);
        player.playSound('armor.equip_chain', { pitch: 0.6, volume: 1 });
    }
});

world.afterEvents.itemUseOn.subscribe(async scb => {
    const { source: player, itemStack: item, block } = scb;
    const loc = block.location;
    const dimension = player.dimension;
    const anyPlr = players.some(player => player.hasTag(`${loc.x} ${loc.y} ${loc.z}`));
    const items = ['scb:block_reinforcer', 'scb:remover', 'scb:mine_injector', 'scb:turret_item', 'scb:dron_item'];
    if (item.typeId == items[0] && anyPlr == false && block.typeId !== 'minecraft:bedrock') {
        player.addTag(`${loc.x} ${loc.y} ${loc.z}`);
        player.playSound('armor.equip_chain', { pitch: 0.6, volume: 1 });
    }
    if (item.typeId == items[1] && player.hasTag(`${loc.x} ${loc.y} ${loc.z}`)) {
        player.removeTag(`${loc.x} ${loc.y} ${loc.z}`);
        player.playSound('block.stonecutter.use');
        const removedBlock = player.dimension.getBlock(loc);
        const returnBlock = new ItemStack(removedBlock.typeId);
        dimension.spawnItem(returnBlock, loc);
        dimension.fillBlocks(loc, loc, 'minecraft:air');
    }
    if (item.typeId == items[2]) {
        const inv = player.getComponent('inventory').container;
        const newAmount = item.amount - 1;
        if (newAmount > 0) {
            const clearMine = new ItemStack(item.typeId, newAmount);
            inv.setItem(player.selectedSlot, clearMine);
            dimension.spawnEntity('scb:mine_entity2', loc);
            player.sendMessage(`§bMine placed at: ${loc.x} ${loc.y} ${loc.z}`)
        }
        else {
            player.runCommand(`replaceitem entity @s slot.weapon.mainhand 0 air`);
            dimension.spawnEntity('scb:mine_entity2', loc);
            player.sendMessage(`§bMine placed at: ${loc.x} ${loc.y} ${loc.z}`)
        }
    }
    if (item.typeId == items[3]) {
        const inv = player.getComponent('inventory').container;
        const newAmount = item.amount - 1;
        if (newAmount > 0) {
            const clearTurret = new ItemStack(item.typeId, newAmount);
            inv.setItem(player.selectedSlot, clearTurret);
            const newTurret = dimension.spawnEntity('scb:turret', new Vector(loc.x + 0.5, loc.y + 1, loc.z + 0.5));
            newTurret.addTag(`owner:${player.name}`);
            player.sendMessage("§bDon't forget to tame me!");
        }
        else {
            player.runCommand(`replaceitem entity @s slot.weapon.mainhand 0 air`);
            const newTurret = dimension.spawnEntity('scb:turret', new Vector(loc.x + 0.5, loc.y + 1, loc.z + 0.5));
            newTurret.addTag(`owner:${player.name}`);
            player.sendMessage("§bDon't forget to tame me!");
        }
    }
    if (item.typeId == items[4]) {
        player.runCommand(`replaceitem entity @s slot.weapon.mainhand 0 air`);
        const newTurret = dimension.spawnEntity('scb:drone', new Vector(loc.x + 0.5, loc.y + 1, loc.z + 0.5));
    }
});


// //get owner
// system.runInterval(() => {
//     for (const player of world.getPlayers()) {
//         const blockOwner = player.getBlockFromViewDirection({ maxDistance: 7 });
//         const entityOwner = player.getEntitiesFromViewDirection({ maxDistance: 7 });
//         if (blockOwner) {
//             const blockObject = blockOwner.block.typeId;
//             if (blockObject.includes('scb:')) {
//                 const blockOwnerName = blockOwner.block.permutation.getAllStates();
//                 console.warn(JSON.stringify(blockOwnerName))
//             }
//         }
//     }
// });